const {
	app, BrowserWindow, Tray, Menu, MenuItem, globalShortcut, ipcMain, shell, clipboard,
	net, session, Notification,
} = require('electron');
const _ = require('lodash');
const path = require('path');
const fs = require('fs');
const os = require('os');
const { exec } = require('child_process');
const Store = require('electron-store');
const windowStateKeeper = require('electron-window-state');
const axios = require('axios');
const https = require('https');

process.env['ELECTRON_DISABLE_SECURITY_WARNINGS'] = true;

if (app.commandLine.hasSwitch('data-path')) {
	const path = app.commandLine.getSwitchValue('data-path');
	app.setPath('userData', path);
	app.setPath('sessionData', path);
}

const store = new Store();
const ICONS_PATH = path.join(__dirname, 'images');
const DEF_ICON = 'favicon.png';
const URL_SUFFIX = '/phone/history?allowedModules=phone';
const URL_ONLY_PHONE_SUFFIX = '?allowedModules=phone';
const DEFAULT_WIDTH = 300;
const DEFAULT_HEIGHT = 600;
const MAX_WIDTH = 600;

let appState = {
	calltrayURL: store.has('lbadminURL') ? store.get('lbadminURL') : '',
	calltrayURLprevious: null,
	connected: false,
	popups: {},
	keyboardShortcut: null,
	notifications: {},
	alwaysOnTop: store.has('alwaysOnTop') ? (store.get('alwaysOnTop') === 'true') : true,
};

const createWindow = (win) => {
	win.setVisibleOnAllWorkspaces(true);

	if (_.isEmpty(appState.calltrayURL)) {
		win.loadFile(path.join(__dirname, 'index.html'));
	} else {

		win.loadURL(formatServerURL(appState.calltrayURL))
			.catch(() => {
				win.loadFile(path.join(__dirname, 'index.html'));
			});
	}

	if (app.commandLine.hasSwitch('debug')) {
		win.webContents.openDevTools();
	};

	win.once('ready-to-show', () => {
		win.show();
	});

	// set allowed modules to local storage
	win.webContents.executeJavaScript(`localStorage.setItem('lbadmin-allowed-modules', 'phone');`, true);
};

const formatServerURL = (url) => {
	if (/^https?:\/\//.test(url)) {
		if (url.endsWith(URL_SUFFIX)) {
			return url;
		} else {
			return '';
		}
	}
	return 'https://' + url + URL_SUFFIX;
};

// try import settings from CallTray 2 for Windows
const tryImportSettings = () => {
	if (appState.calltrayURL || os.platform() !== 'win32') {
		return;
	}
	const oldSettingsFile = 'C:\\Program Files (x86)\\CallTray\\calltray\\package.json';
	try {
		if (!fs.existsSync(oldSettingsFile)) {
			return;
		}
		const oldSettingsJson = fs.readFileSync(oldSettingsFile, 'utf8');
		const oldSettings = JSON.parse(oldSettingsJson);
		if (oldSettings && oldSettings.main) {
			appState.calltrayURL = oldSettings.main;
		}
	} catch (err) {
	}
};
tryImportSettings();

app.commandLine.appendSwitch('ignore-certificate-errors');
app.disableHardwareAcceleration();
app.whenReady().then(() => {
	if (process.platform === 'win32') {
		app.setAppUserModelId('CallTray');
	}
	const mainWindowState = windowStateKeeper({
		defaultWidth: DEFAULT_WIDTH,
		defaultHeight: DEFAULT_HEIGHT,
	});
	const mainWindow = new BrowserWindow({
		title: 'CallTray',
		icon: path.join(ICONS_PATH, 'favicon48.png'),
		show: true,
		frame: false,
		movable: true,
		transparent: true,
		alwaysOnTop: appState.alwaysOnTop,
		skipTaskbar: true,
		x: mainWindowState.x,
		y: mainWindowState.y,
		width: mainWindowState.width,
		height: mainWindowState.height,
		minWidth: DEFAULT_WIDTH,
		minHeight: DEFAULT_HEIGHT,
		maxWidth: MAX_WIDTH,
		webPreferences: {
			preload: path.join(__dirname, 'preload.js'),
		},
		resizable: false,
		maximizable: false,
	});
	createWindow(mainWindow);
	mainWindowState.manage(mainWindow);

	const showMainWindow = (focus) => {
		mainWindow.show();
		mainWindow.setVisibleOnAllWorkspaces(true);
		mainWindow.setAlwaysOnTop(appState.alwaysOnTop);
		if (focus) {
			mainWindow.focus();
		}

		mainWindow.webContents.send('showPopup');
	};

	const hideMainWindow = () => {
		if (!mainWindow.skipTaskbar) {
			mainWindow.minimize();
		} else {
			mainWindow.hide();
		}

		mainWindow.webContents.send('hidePopup');
	};

	const isMainWindowVisible = () => !mainWindow.isMinimized();

	const toggleMainWindow = () => {
		if (isMainWindowVisible()) {
			hideMainWindow();
		} else {
			showMainWindow();
		}
	};

	let tray = new Tray(path.join(ICONS_PATH, DEF_ICON));

	tray.on('click', () => {
		toggleMainWindow();
	});

	let ctxMenu = new Menu();
	ctxMenu.append(new MenuItem({
		label: 'Zobrazit/skrýt',
		click: () => {
			toggleMainWindow();
		},
	}));
	ctxMenu.append(new MenuItem({
		label: 'Znovu načíst',
		click: () => {
			showMainWindow();
			mainWindow.reload();
		},
	}));
	ctxMenu.append(new MenuItem({
		label: 'Změnit URL serveru',
		click: () => {
			mainWindow.webContents.send('logout');
			setTimeout(() => {
				showMainWindow();
				appState.calltrayURLprevious = appState.calltrayURL;
				appState.calltrayURL = '';
				createWindow(mainWindow);
			}, 500);
		},
	}));
	ctxMenu.append(new MenuItem({
		label: 'Vždy nahoře',
		type: 'checkbox',
		checked: appState.alwaysOnTop,
		click: () => {
			if (appState.alwaysOnTop !== ctxMenu.items[3].checked) {
				appState.alwaysOnTop = ctxMenu.items[3].checked;
				mainWindow.setAlwaysOnTop(appState.alwaysOnTop);
				store.delete('alwaysOnTop');
				store.set('alwaysOnTop', appState.alwaysOnTop.toString());
			}
		},
	}));
	ctxMenu.append(new MenuItem({
		label: 'O aplikaci',
		enabled: false,
		click: () => {
			let url = store.has('lbadminURL') ? store.get('lbadminURL') : '';
			url = formatServerURL(url).replace(URL_SUFFIX, '/phone/about');;
			openPopup(null, 'phone-about', url, 280, 600, false);
		},
	}));
	ctxMenu.append(new MenuItem({
		label: 'Ukončit',
		click: () => { app.quit(); },
	}));
	tray.setContextMenu(ctxMenu);

	let prevClpContent = clipboard.readText();
	setInterval(() => {
		let clpContent = clipboard.readText();
		if (clpContent !== prevClpContent) {
			prevClpContent = clpContent;
			mainWindow.webContents.send('clipboardChange', clpContent);
		}
	}, 500);

	mainWindow.on('focus', () => {
		mainWindow.webContents.send('focus');
	});
	mainWindow.on('blur', () => {
		mainWindow.webContents.send('blur');
	});

	// if on login page, disable "About" page
	mainWindow.webContents.on('did-stop-loading', ()=> {
		let url = mainWindow.webContents.getURL();
		ctxMenu.items[4].enabled = (!url.endsWith('/login') && !url.startsWith('file://'));
		tray.setContextMenu(ctxMenu);
	});

	ipcMain.on('isTransparent', (event) => {
		event.returnValue = mainWindow.transparent;
	});

	ipcMain.on('connectInternal', (event) => {
		appState.connected = true;
		mainWindow.webContents.send('connect');
	});

	ipcMain.on('disconnectInternal', (event) => {
		appState.connected = false;
		mainWindow.webContents.send('disconnect');
	});

	ipcMain.on('setServerURL', (event, url) => {
		if (url) {
			appState.calltrayURL = url;
			createWindow(mainWindow);
			event.returnValue = appState.calltrayURL;
			store.delete('lbadminURL');
			store.set('lbadminURL', appState.calltrayURL);
		}
	});

	ipcMain.on('showMainWindow', (event, focus) => {
		showMainWindow();
	});

	ipcMain.on('hideMainWindow', (event) => {
		hideMainWindow();
	});

	ipcMain.on('isMainWindowVisible', (event) => {
		event.returnValue = !mainWindow.isMinimized();
	});

	ipcMain.on('resizableMainWindow', (event, resizable) => {
		mainWindow.setResizable(resizable);
	});

	ipcMain.on('isConnected', (event) => {
		event.returnValue = appState.connected;
	});

	ipcMain.on('registerShortcut', (event, shortcut) => {
		if (shortcut !== appState.keyboardShortcut) {
			globalShortcut.unregisterAll();
			try {
				const ret = globalShortcut.register(shortcut, () => {
					mainWindow.webContents.send('shortcutPress');
				});
				appState.keyboardShortcut = shortcut ? ret : appState.keyboardShortcut;
			} catch (e) {}
		}
	});

	ipcMain.on('changeIcon', (event, icon, title) => {
		tray.setImage(path.join(ICONS_PATH, icon));
		tray.setToolTip(title);
	});

	const openPopup = function (event, name, url, width, height, frame) {
		let xPosition = 0;
		if ((mainWindowState.x - width - 10) > 0) {
			xPosition = mainWindowState.x - width - 10;
		} else {
			xPosition = mainWindowState.x + mainWindowState.width + 10;
		}
		let options = {
			width: width,
			height: height,
			frame: false,
			transparent: true,
			alwaysOnTop: false,
			skipTaskbar: true,
			minWidth: width,
			minHeight: height,
			webPreferences: {
				preload: path.join(__dirname, 'preload.js'),
			},
			resizable: false,
			x: xPosition,
			y: mainWindowState.y,
			maximizable: false,
		};

		if (name === 'phone-selfcare') {
			const bounds = store.get('selfcare_bounds');
			if (bounds) {
				options.x = bounds.x;
				options.y = bounds.y;
				options.width = bounds.width;
				options.height = bounds.height;
			}
		}

		if (frame) {
			options.frame = true;
			options.skipTaskbar = false;
			options.transparent = false;
		}

		// not opening two same popups
		if (appState.popups[name]?.win?.isVisible()) return;

		appState.popups[name] = {
			win: new BrowserWindow(options),
		};
		appState.popups[name].win.setVisibleOnAllWorkspaces(true);
		appState.popups[name].win.setMenuBarVisibility(false);
		appState.popups[name].win.setAlwaysOnTop(appState.alwaysOnTop);
		appState.popups[name].win.loadURL(url + URL_ONLY_PHONE_SUFFIX);

		// open dev tools for popup window if DEBUG switch
		if (app.commandLine.hasSwitch('debug')) {
			appState.popups[name].win.webContents.openDevTools();
		};

		if (name === 'phone-selfcare') {
			appState.popups[name].win.setResizable(true);
		}
	};

	ipcMain.on('openPopup', openPopup);

	ipcMain.on('closePopup', (event, name) => {
		if (!(name in appState.popups)) {
			return;
		}
		if (name === 'phone-selfcare') {
			store.delete('selfcare_bounds');
			store.set('selfcare_bounds', appState.popups[name].win.getBounds());
		}
		appState.popups[name].win.close();
		delete appState.popups[name];
	});

	ipcMain.on('getVersion', (event) => {
		event.returnValue = app.getVersion();
	});

	ipcMain.on('getServerURL', (event) => {
		event.returnValue = appState.calltrayURL;
	});

	ipcMain.on('getPreviousServerURL', (event) => {
		event.returnValue = appState.calltrayURLprevious;
	});

	ipcMain.on('openExternalBrowser', (event, url) => {
		shell.openExternal(url);
	});

	ipcMain.on('openExternalApplication', (event, command) => {
		exec(command, (err) => {
			if (err) {
				console.log('exec error', err);
			}
		});
	});

	ipcMain.on('formatServerURL', (event) => {
		event.returnValue = formatServerURL(appState.calltrayURL);
	});

	ipcMain.handle('checkServer', async (event, url) => {
		if (!url) {
			return false;
		}
		let isLBAdmin = false;
		const serverURL = formatServerURL(url).replace(URL_SUFFIX, '/api/lbadmin/lbadmin/user');
		try {
			const response = await axios.get(serverURL, {
				timeout: 5000,
				httpsAgent: new https.Agent({
					rejectUnauthorized: false
				})
			});
			if (response.status === 200 && 'user_name' in response.data) {
				isLBAdmin = true;
			}
		} catch (err) {
		}
		return isLBAdmin;
	});

	// windows/linux/mac
	ipcMain.on('getPlatform', (event) => {
		const platform = os.platform();
		if (platform === 'win32') {
			event.returnValue = 'windows';
		} else if (platform === 'darwin') {
			event.returnValue = 'mac';
		} else {
			event.returnValue = platform;
		}
	});

	ipcMain.on('showNotification', (event, text, title, tag) => {
		// close/remove previous notification with same tag
		if (appState.notifications[tag]) {
			appState?.notifications[tag]?.close();
			delete appState.notifications[tag];
		}

		appState.notifications[tag] = new Notification({
			title: title || 'LBPhone',
			body: text,
			icon: path.join(ICONS_PATH, 'favicon48.png')
		});

		// on notification click show main window
		appState.notifications[tag].on('click', (event, arg) => {
			showMainWindow();
		});

		// on notification close, remove from notifications
		appState.notifications[tag].on('close', (event, arg) => {
			delete appState.notifications[tag];
		});

		appState.notifications[tag].show();
	});

	// close notification
	ipcMain.on('hideNotification', (event, tag) => {
		appState?.notifications[tag]?.close();
		delete appState.notifications[tag];
	});

	ipcMain.on('emitMessage', (event, message) => {
		mainWindow.webContents.send('msg-' + message);
	});

	// enable eval for templates in incoming actions
	session.defaultSession.webRequest.onHeadersReceived((details, callback) => {
		/* eslint-disable-next-line standard/no-callback-literal */
		callback({
			responseHeaders: {
				...details.responseHeaders,
				'Content-Security-Policy': [
					'default-src \'self\' data: \'unsafe-eval\' \'unsafe-inline\'',
				],
			},
		});
	});
});

app.on('will-quit', () => {
	globalShortcut.unregisterAll();
});
